@@ -1,12 +1,37 @@
+#!/usr/bin/env python
+
 __doc__ = """
+
 This program requires python 3.6 or higher.
 
-print(functionName.__doc__) to see the
+This module, MissingHeavyAtoms.py has all 
 
-documentation for the function or usage.
+the functions required to fix missing heavy
+
+atoms. These functions represent the 20 common 
+
+amino acid residues and other extra functions.
 
-Send enquiries to osita@protein-science.com
 """
+
+__author__     = "Osita Sunday Nnyigide"
+
+__copyright__  = "Copyright 2022, Osita Sunday Nnyigide"
+
+__credits__    = ["Tochukwu Olunna Nnyigide", "Lee Sun-Gu", "Hyun Kyu"]
+
+__license__    = "MIT"
+
+__version__    = "1.0.7"
+
+__maintainer__ = "Osita Sunday Nnyigide"
+
+__email__      = "osita@protein-science.com"
+
+__status__     = "Production"
+
+__date__       = "May 11, 2022"
+
 import numpy as np
 from numpy import cross, dot
 from math import sqrt, acos,degrees, pi
@@ -49,11 +74,12 @@ def calcCoordinate(a, b, c, bond_len, theta, di_angle):
     A list: the calculated 4th atom coordinate
     """
 
-    a,b,c = np.array(a),np.array(b),np.array(c)
+    a,b,c    = np.array(a),np.array(b),np.array(c)
     di_angle = di_angle*pi/180.0
 
     u = c - b
     x = a - b
+
     v = ((x) - (np.dot((x), u)/np.dot(u,u))*u)
 
     w = np.cross(u, x)
@@ -65,18 +91,22 @@ def calcCoordinate(a, b, c, bond_len, theta, di_angle):
 
     u1 = b - c
     y1 = pos_temp2 - c
+
     mag_y1  = sqrt((y1[0]**2)+(y1[1]**2)+(y1[2]**2))
     mag_u1  = sqrt((u1[0]**2)+(u1[1]**2)+(u1[2]**2))
 
     theta_bcd = acos(np.dot(u1,y1)/(mag_u1*mag_y1))
-    rotate = theta - degrees(theta_bcd)
+    rotate    = theta - degrees(theta_bcd)
 
     z  = np.cross(u1, y1)
     n  = z/sqrt((z[0]**2)+(z[1]**2)+(z[2]**2))
+
     pos_ini = c + y1*np.cos(rotate*pi/180) +\
-            (np.cross(n,y1)*np.sin(rotate*pi/180))\
-            + n*(np.dot(n,y1))*(1-np.cos(rotate*pi/180))
+              (np.cross(n,y1)*np.sin(rotate*pi/180))\
+              + n*(np.dot(n,y1))*(1-np.cos(rotate*pi/180))
+
     pos_BL = ((pos_ini-c)*(bond_len/np.linalg.norm(pos_ini-c)))+c
+
     return pos_BL.tolist()
 
 def calcTorsionAngle(coord1, coord2, coord3, coord4):
@@ -99,19 +129,25 @@ def calcTorsionAngle(coord1, coord2, coord3, coord4):
 
     coord1,coord2,coord3,coord4 = np.array(coord1), \
     np.array(coord2),np.array(coord3),np.array(coord4)
+
     bvec12 = coord1 - coord2
     bvec32 = coord3 - coord2
     bvec43 = coord4 - coord3
+
     pvec13 = cross(bvec12, bvec32)
     pvec24 = cross(bvec43, bvec32)
+
     projec = dot(pvec13, pvec24)
     sqrd13 = dot(pvec13, pvec13)
     sqrd24 = dot(pvec24, pvec24)
+
     cosine = projec / sqrt(sqrd13*sqrd24)
     cosine = min(1.0, max(-1.0, cosine))
     radian = acos(cosine)
+
     if dot(pvec13, cross(pvec24, bvec32)) < 0:
         radian = -radian
+        
     return degrees(radian)
 
 def get_chi(atom_1, atom_2, atom_3, atom_4, resNum, chainNum,faspr):
@@ -146,6 +182,7 @@ def get_chi(atom_1, atom_2, atom_3, atom_4, resNum, chainNum,faspr):
                     pos2 = atmpos[i][atom[i].index(atom_2)]
                     pos3 = atmpos[i][atom[i].index(atom_3)]
                     pos4 = atmpos[i][atom[i].index(atom_4)]
+
                     return calcTorsionAngle(pos1,pos2,pos3,pos4)
 
 def addBackbone(N, CA, C, atmpos, psi):
@@ -189,6 +226,7 @@ def addBackbone(N, CA, C, atmpos, psi):
             O = calcCoordinate(N,CA,C,1.23,120.5,i)
             if calcDistance(O,atmpos[0]) > 2.0:
                 break
+                
     return O
 
 def c_termini(atom,pos):
@@ -656,7 +694,7 @@ def repairGln(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
     atoms of the GLN residue.
     See repairSer() for full annotation
     """
-
+    
     N  = coord[atoms.index('N')]
     CA = coord[atoms.index('CA')]
     C  = coord[atoms.index('C')]
@@ -724,6 +762,12 @@ def repairGln(atoms,coord,resNo,chain,missing,faspr,nextres='',psi=''):
             return [atoms, atompos]
 
 
+    CG  = coord[atoms.index('CG')]
+
+    CD  = coord[atoms.index('CD')]
+
+    OE1 = coord[atoms.index('OE1')]
+
     CB_CG_CD_OE1_diangle = calcTorsionAngle(CB,CG,CD,OE1)
 
     CB_CG_CD_NE2_diangle = 180.0 + CB_CG_CD_OE1_diangle
