@@ -1,12 +1,37 @@
+#!/usr/bin/env python
+
 __doc__ = """
+
 This program requires python 3.6 or higher.
 
-print(functionName.__doc__) to see the
+This module, MissingHydrogenAtoms.py has all 
 
-documentation for the function or usage.
+the functions required to fix missing hydrogen
+
+atoms. These functions represent the 20 common 
+
+amino acid residues and other extra functions.
 
-Send enquiries to osita@protein-science.com
 """
+
+__author__     = "Osita Sunday Nnyigide"
+
+__copyright__  = "Copyright 2022, Osita Sunday Nnyigide"
+
+__credits__    = ["Tochukwu Olunna Nnyigide", "Lee Sun-Gu", "Hyun Kyu"]
+
+__license__    = "MIT"
+
+__version__    = "1.0.7"
+
+__maintainer__ = "Osita Sunday Nnyigide"
+
+__email__      = "osita@protein-science.com"
+
+__status__     = "Production"
+
+__date__       = "May 11, 2022"
+
 import copy
 from . import LinearAlgebra
 from math import sqrt
@@ -1126,17 +1151,18 @@ def get_hd1(ires):
     pos_CE1    = coord[atoms.index('CE1')]
 
     HD1        =  LinearAlgebra.class5(pos_CE1,pos_ND1,pos_CG,1.01)
+
     return [HD1]
 
 def prot_his(x,res_pos,atom_name,chainNo):
     """
     This function checks if there are more than 4 HIS resi in order to add
-    # hydrogen atom to ND1 of histidine making it to have +1 charge.
+    hydrogen atom to ND1 of histidine making it to have +1 charge.
 
     ----------
-    x:       a list containing all residue name,atoms,atom coordinates
+    x:         a list containing all residue name,atoms,atom coordinates
 
-    res_pos: a list containing all residue atom positions
+    res_pos:   a list containing all residue atom positions
 
     atom_name: a list containing all residue atom names
 
@@ -1144,15 +1170,17 @@ def prot_his(x,res_pos,atom_name,chainNo):
 
     Returns
     -------
-    A list:   index 0 = updated res_pos and index 1 = updated atom_name
+    A list:     index 0 = updated res_pos and index 1 = updated atom_name
     """
     his  = [j for i,j in enumerate(x) if x[i][0][:3] == 'HIS']
     num  = [res_pos[i] for i,j in enumerate(x) if x[i][0][:3] == 'HIS']
     name = [atom_name[i] for i,j in enumerate(x) if x[i][0][:3] == 'HIS']
+
     if len(his) <= 4:
         pass
     else:
         prot = len(his)//5
+
         for i in range(prot):
             res_pos[res_pos.index(num[i])].extend(get_hd1(his[i]))
             atom_name[atom_name.index(name[i])].extend(['HD1'])
