@@ -1,12 +1,37 @@
+#!/usr/bin/env python
+
 __doc__ = """
+
 This program requires python 3.6 or higher.
 
-print(functionName.__doc__) to see the 
+This module, PotentialEnergy.py has the function(s)
 
-documentation for the function or usage.
+that is used to calculate the potential energy between 
+
+class6 hydrogen atoms with rotational freedom and the
+
+rest of protein heavy atoms.
 
-Send enquiries to osita@protein-science.com
 """
+
+__author__     = "Osita Sunday Nnyigide"
+
+__copyright__  = "Copyright 2022, Osita Sunday Nnyigide"
+
+__credits__    = ["Tochukwu Olunna Nnyigide", "Lee Sun-Gu", "Hyun Kyu"]
+
+__license__    = "MIT"
+
+__version__    = "1.0.7"
+
+__maintainer__ = "Osita Sunday Nnyigide"
+
+__email__      = "osita@protein-science.com"
+
+__status__     = "Production"
+
+__date__       = "May 11, 2022"
+
 import numpy as np
 from numpy import cross, dot
 from .ForcefieldParam import param
@@ -25,6 +50,7 @@ def calcDistance(m, n):
     -------
     Float: the calculated distance
     """
+
     return (sqrt(sum([(m[z] - n[z])**2 for z in range(3)])))/10
 
 def recalcCoordinate(b,c,d,bond_len):
@@ -43,21 +69,28 @@ def recalcCoordinate(b,c,d,bond_len):
     -------
     A list: the calculated coordinate
     """
+
     b,c,d = np.array(b),np.array(c),np.array(d)
     theta = 107.0 
+
     u1 = b - c
     y1 = d - c
+
     mag_y1  = sqrt((y1[0]**2)+(y1[1]**2)+(y1[2]**2))
     mag_u1  = sqrt((u1[0]**2)+(u1[1]**2)+(u1[2]**2))
 
     theta_bcd = acos(np.dot(u1,y1)/(mag_u1*mag_y1))
-    rotate = theta - degrees(theta_bcd)
+    rotate    = theta - degrees(theta_bcd)
 
     z  = np.cross(u1, y1)
     n  = z/sqrt((z[0]**2)+(z[1]**2)+(z[2]**2))
-    pos_ini = c + y1*np.cos(rotate*pi/180) +(np.cross(n,y1)*np.sin(rotate*pi/180))\
-     + n*(np.dot(n,y1))*(1-np.cos(rotate*pi/180))
-    pos_BL = ((pos_ini-c)*(1.09/np.linalg.norm(pos_ini-c)))+c
+
+    pos_ini = c + y1*np.cos(rotate*pi/180) \
+              +(np.cross(n,y1)*np.sin(rotate*pi/180))\
+              + n*(np.dot(n,y1))*(1-np.cos(rotate*pi/180))
+
+    pos_BL = ((pos_ini-c)*(0.96/np.linalg.norm(pos_ini-c)))+c
+    
     return pos_BL.tolist()
 
 def calcCoordinate(a,b,c,bond_len,di_angle,theta):
@@ -79,7 +112,8 @@ def calcCoordinate(a,b,c,bond_len,di_angle,theta):
     -------
     A list: the calculated 4th atom coordinate
     """
-    a,b,c = np.array(a),np.array(b),np.array(c)
+
+    a,b,c    = np.array(a),np.array(b),np.array(c)
     di_angle = di_angle*pi/180.0
 
     u = c - b
@@ -95,17 +129,22 @@ def calcCoordinate(a,b,c,bond_len,di_angle,theta):
     
     u1 = b - c
     y1 = d - c
+
     mag_y1  = sqrt((y1[0]**2)+(y1[1]**2)+(y1[2]**2))
     mag_u1  = sqrt((u1[0]**2)+(u1[1]**2)+(u1[2]**2))
 
     theta_bcd = acos(np.dot(u1,y1)/(mag_u1*mag_y1))
-    rotate = theta - degrees(theta_bcd)
+    rotate    = theta - degrees(theta_bcd)
 
     z  = np.cross(u1, y1)
     n  = z/sqrt((z[0]**2)+(z[1]**2)+(z[2]**2))
-    pos_ini = c + y1*np.cos(rotate*pi/180) +(np.cross(n,y1)*np.sin(rotate*pi/180))\
-     + n*(np.dot(n,y1))*(1-np.cos(rotate*pi/180))
+
+    pos_ini = c + y1*np.cos(rotate*pi/180) \
+              +(np.cross(n,y1)*np.sin(rotate*pi/180))\
+              + n*(np.dot(n,y1))*(1-np.cos(rotate*pi/180))
+
     pos_BL = ((pos_ini-c)*(bond_len/np.linalg.norm(pos_ini-c)))+c
+
     return pos_BL.tolist()
 
 def _optmize(pos_CA,pos_CB,pos_OG1,pos_HG1,BL,di):
@@ -125,9 +164,11 @@ def _optmize(pos_CA,pos_CB,pos_OG1,pos_HG1,BL,di):
     -------
     A list: the coordinate corresponding to H after each rotation 
     """
+
     CB_OG1_HG1_angle = 109.5
     opt_h = []
-    for i in range(0, 360, 45):
+
+    for i in range(0, 360, 30):
         point_h = calcCoordinate(pos_CA,pos_CB,pos_OG1,BL,CB_OG1_HG1_angle,di)
         point_h = recalcCoordinate(pos_CB,pos_OG1,point_h,BL)
         opt_h.extend([point_h])
@@ -156,6 +197,7 @@ def optmizeH(num,atom,atmpos,resNo,resn,data):
     -------
     A list: the coordinate corresponding to the lowest potential energy 
     """
+    
     total_ener = []
     opt_h = _optmize(data[0],data[1],data[2],data[3],data[4],data[5])
     for z in range(len(resNo)):
@@ -167,7 +209,7 @@ def optmizeH(num,atom,atmpos,resNo,resn,data):
                 poten_ener = 0
                 for k,l in enumerate(_pos):
                     for n,m in enumerate(_pos[k]):
-                        if sqrt(sum([(h[q]-m[q])**2 for q in range(3)])) <= 3.5:
+                        if sqrt(sum([(h[q]-m[q])**2 for q in range(3)])) <= 2.5:
                             r_die     = 138.94
                             H_pcharge = data[6]
                             A_pcharge = param[_res[k]][_name[k][n]][0]
