@@ -1,12 +1,34 @@
+#!/usr/bin/env python
+
 __doc__ = """
+
 This program requires python 3.6 or higher.
 
-print(functionName.__doc__) to see the 
+This module, SecondaryStructure.py has the function
 
-documentation for the function or usage.
+that is used to assign secondary structure elements.
 
-Send enquiries to osita@protein-science.com
 """
+
+__author__     = "Osita Sunday Nnyigide"
+
+__copyright__  = "Copyright 2022, Osita Sunday Nnyigide"
+
+__credits__    = ["Tochukwu Olunna Nnyigide", "Lee Sun-Gu", "Hyun Kyu"]
+
+__license__    = "MIT"
+
+__version__    = "1.0.7"
+
+__maintainer__ = "Osita Sunday Nnyigide"
+
+__email__      = "osita@protein-science.com"
+
+__status__     = "Production"
+
+__date__       = "May 11, 2022"
+
+import sys
 import itertools
 from . import ReadStructure
 import matplotlib.pyplot as plt
@@ -14,8 +36,8 @@ import matplotlib.pyplot as plt
 def assignStructure(filename):
 	"""
 	This program assigns secondary structure elements and
-	writes all data of structure assignment 
-	in a text file and plots the value in TIF format
+	writes all data of structure assignment in a text file 
+	and plots the value in TIF format
 
 	This funtion & the dependency (ReadStructure.py) read PRAS 
 	output PDB file in a quick and complex manner. Since it is a 
@@ -29,34 +51,66 @@ def assignStructure(filename):
 
 	Arguments
 	----------
-	filename: PRAS generated PDB file
+	filename: PRAS generated PDB structure file (.pdb or .cif)
 
 	Returns
 	-------
-	None: writes a text file containing detailed secondary structure and plots a figure  
+	None:     writes a text file containing detailed secondary 
+	          structure and plots a figure  in TIF format
 	"""
-	lines = open(filename, 'r').readlines()
-	rf = [i.strip("\n") for i in lines if i[:4] == 'ATOM' or i[0:3] == "TER"]
-	chains = [i for i,j in enumerate(rf) if j[:3].strip(' ')  == "TER"]
+
+	_format = filename[-4:]
+
+	if _format == '.cif':
+		try:
+			lines = open(filename, 'r').readlines()
+			rf              = [i.strip("\n") for i in lines if i[:4] == 'ATOM']
+			chains          = list(dict.fromkeys([i.split()[6] for i in rf]))
+			atoms_per_chain = [[j for j in rf if j.split()[6] == chains[i]]\
+							  for i in range(len(chains))]
+		except:
+			print('{} could not be read, check the file and format.'
+				 '\n' 'Perhaps it should be a pdb but you specified cif.'
+				  ' Secondary''\n''structure could not be assigned'.format(filename))
+			sys.exit(1)			
+
+	elif _format == '.pdb':
+		try:
+			lines   = open(filename, 'r').readlines()
+			rf      = [i.strip("\n") for i in lines if i[:4] == 'ATOM' or i[0:3] == "TER"]
+			chains  = [i for i,j in enumerate(rf) if j[:3].strip(' ')  == "TER"]
+		except:
+			print('{} could not be read, check the file and format.'
+				 '\n' 'Perhaps it should be a cif but you specified pdb.'
+				  ' Secondary''\n''structure could not be assigned'.format(filename))
+			sys.exit(1)
+
+	else:
+		print('Fatal error! Please check file type')
+		sys.exit(1)
 
 	count = 0
 	for n in range(len(chains)):
-		line = [i for i in list(itertools.chain(*[rf[count:i]for i,j in enumerate(rf) if i == chains[n]]))]
+		chain_i = [i for i in list(itertools.chain(*[rf[count:i]for i,j in enumerate(rf) if i == chains[n]]))]\
+			   if filename[-4:] != '.cif' else atoms_per_chain[n]
 
-		ahelix_hb    = ReadStructure.alpha_helix(line)
-		amide_dih    = ReadStructure.amideDihedral(line)
-		_310helix_hb = ReadStructure._310helix(line)
-		bturn_nhb    = ReadStructure.beta_turn(line)
-		phi_dih      = ReadStructure.phiDihedral(line)
-		psi_dih      = ReadStructure.psiDihedral(line)
-		pihelix_hb   = ReadStructure.pi_helix(line)
-		nextres      = ReadStructure.nextRes(line)
-		chain        = ReadStructure.resChain(line)
-		resn         = ReadStructure.resName(line)
-		resNo        = ReadStructure.resNumber(line)
+		ahelix_hb    = ReadStructure.alpha_helix(chain_i, _format)
+		amide_dih    = ReadStructure.amideDihedral(chain_i, _format)
+		_310helix_hb = ReadStructure._310helix(chain_i, _format)
+		bturn_nhb    = ReadStructure.beta_turn(chain_i,  _format)
+		phi_dih      = ReadStructure.phiDihedral(chain_i, _format)
+		psi_dih      = ReadStructure.psiDihedral(chain_i, _format)
+		pihelix_hb   = ReadStructure.pi_helix(chain_i, _format)
 
-		"""initially assign pi helix, alpha helix, 310 helix and the rest Coil
-			using the recognition criterion as described in the PRAS paper"""
+		nextres      = ReadStructure.nextRes(chain_i, _format) 
+		chain        = ReadStructure.resChain(chain_i, _format)
+		resn         = ReadStructure.resName(chain_i, _format)
+		resNo        = ReadStructure.resNumber(chain_i, _format)
+
+		"""
+		initially assign pi helix, alpha helix, 310 helix and the rest Coil
+		using the recognition criterion as described in the PRAS paper
+		"""
 		v = list(zip(ahelix_hb,_310helix_hb,amide_dih,bturn_nhb,phi_dih,psi_dih,pihelix_hb))
 		SSA = []
 		for i in range(len(v)):
@@ -82,8 +136,10 @@ def assignStructure(filename):
 			else:
 				SSA.append('Coil')
 
-		"""extend each pi helix region by 5,
-		the 5th (first resi) is assigned already"""
+		"""
+		extend each pi helix region by 5,
+		the 5th (first resi) is assigned already
+		"""
 		extend_pihelix = []
 		for i, extend_helix in enumerate(SSA):
 			_helix_dict ={}
@@ -97,8 +153,10 @@ def assignStructure(filename):
 				SSA[value+3]   = 'Pi_helix'
 				SSA[value+4]   = 'Pi_helix'
 
-		"""extend each alpha helix region by 4,
-			the 4th (first resi) is assigned already"""
+		"""
+		extend each alpha helix region by 4,
+		the 4th (first resi) is assigned already
+		"""
 		extend_ahelix = []
 		for i, extend_helix in enumerate(SSA):
 			_helix_dict ={}
@@ -113,8 +171,10 @@ def assignStructure(filename):
 				SSA[value+2]   = 'Alpha_helix'
 				SSA[value+3]   = 'Alpha_helix'
 
-		"""extend each 310helix region by 3,
-			the 3rd (first resi) is assigned already"""
+		"""
+		extend each 310helix region by 3,
+		the 3rd (first resi) is assigned already
+		"""
 		extend_3helix = []
 		for i, extend_310helix in enumerate(SSA):
 			_helix_dict ={}
@@ -160,9 +220,9 @@ def assignStructure(filename):
 		_nhbturn_list = []
 		for i, nhb_beta_turn in enumerate(SSA):
 			_nhbturn_dict ={}
-			if  (v[i][3] <= 4.7) and (SSA[i]!= 'Alpha_helix' and SSA[i] != '310_helix' and SSA[i] != 'Pi_helix' and SSA[i] != 'Turn_hb'and SSA[i] != 'Strand')\
-				and (SSA[(i+1)% len(SSA)] != 'Alpha_helix' and SSA[(i+1)% len(SSA)] != 'Pi_helix' and SSA[(i+1)% len(SSA)]\
-				!= '310_helix') and len(SSA[i:])>2:
+			if  (v[i][3] <= 4.7) and (SSA[i]!= 'Alpha_helix' and SSA[i] != '310_helix' and SSA[i] != 'Pi_helix' and SSA[i]\
+				!= 'Turn_hb'and SSA[i] != 'Strand') and (SSA[(i+1)% len(SSA)] != 'Alpha_helix' and SSA[(i+1)% len(SSA)] \
+				!= 'Pi_helix' and SSA[(i+1)% len(SSA)]!= '310_helix') and len(SSA[i:])>2:
 				_nhbturn_dict[nhb_beta_turn] = i
 				_nhbturn_list.append(_nhbturn_dict)
 		for nhb_beta_turn in _nhbturn_list:
@@ -184,7 +244,7 @@ def assignStructure(filename):
 			for key, value in pp_helix.items():
 				SSA[value]     = 'Polyp_helix'
 
-		# count all assigned helices and write to output file
+		# count all assigned SS elements and write to output file
 		total_coil       =  SSA.count('Coil')
 		total_nhb_turn   =  SSA.count('Turn_nhb')
 		total_hb_turn    =  SSA.count('Turn_hb')
@@ -195,9 +255,10 @@ def assignStructure(filename):
 		total_pphelix    =  SSA.count('Polyp_helix')
 		pdb_code         =  filename
 
+
 		with open ('sec_strc_table_'+'chain'+str(n+1)+'.txt', 'w') as f:
 			f.write('When using files generated by this program in a publication, please cite this program as "O.S. Nnyigide, T.O. Nnyigide,'+'\n'
-				'S.G. Lee, K. Hyun. PRAS: A Web Server to Repair PDB Files, Add Missing Heavy Atoms and Hydrogen Atoms and Assign'+'\n' 
+				'S.G. Lee, K. Hyun. PRAS: A Web Server to Repair PDB Structures, Add Missing Heavy Atoms and Hydrogen Atoms and Assign'+'\n' 
 				'Secondary Structure by Amide Interactions. Submitted'+'\n')
 			f.write('#'*118 +'\n');f.write('Secondary Structure Assignment by Amide Interactions.'
 			' This is the summary of the measurements used to assign structure' +'\n');f.write('#'*118+'\n')
@@ -216,8 +277,7 @@ def assignStructure(filename):
 				  'Pp-helix = {0:.1f}%'.format(data[3]), 'β-strand = {0:.1f}%'.format(data[4]), 'Turn = {0:.1f}%'.format(data[5]),\
 			 	  'Coil = {0:.1f}%'.format(data[6])]
 
-			x = []
-			y = []
+			x, y = [], []
 			for i,j in enumerate(SS):
 			  if data[i] != 0.000:
 			    x.append(data[i])
@@ -232,7 +292,8 @@ def assignStructure(filename):
 			plt.savefig('sec_strc_plot_'+'chain'+str(n+1)+'.tif')
 			plt.close(fig)
 
-			if n == 0:
-				count+= chains[n]+1
-			else:
-				count+= (chains[n]-chains[n-1])
\ No newline at end of file
+			if _format == '.pdb':
+				if n == 0:
+					count+= chains[n]+1
+				else:
+					count+= (chains[n]-chains[n-1])
\ No newline at end of file
